---------------------------------------------------------------------
-- File:        Card_Settings.vhd
-- Description: Settings readout and internal storage.
-- Designer:    Christos Zamantzas
-- Date:        2010/02/11 rev.08

-- ver. 04: 23/08/2008 changed order of connection - masking tables
-- ver. 05: 09/10/2008 added second long for the modification date 
-- ver. 06: 17/11/2009 added check for the available CFCs. 
-- ver. 07: 01/02/2010 added new varible for the channel connection.
-- ver. 08: 11/02/2010 updated memory mapping to follow last FESA table.

----------------------------------------------------------------------
-- TODO: 
-- Checking for connected CFCs should be done in a more deterministic and safe way.
--    TBD: use the BISConnect or CableConnected flags (?) 
-- Make needed changes so that count_int variable can become a signal. 
-----------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.all;

-- entity declaration

ENTITY Card_Settings IS
PORT
(
CLK          : IN STD_LOGIC;
Reset        : IN STD_LOGIC;
Enable       : IN STD_LOGIC;                   
Data         : IN STD_LOGIC_VECTOR(7 downto 0); 
Address      : IN STD_LOGIC_VECTOR(5 downto 0); 

Count        : OUT STD_LOGIC_VECTOR(5 downto 0);      -- test output
CableConnect : OUT STD_LOGIC_VECTOR(15 downto 0);     -- Connection Table (Is_Cable Connected)
BISConnect   : OUT STD_LOGIC_VECTOR(15 downto 0);     -- Connection Table (Is_BIS_Connected)
Masking      : OUT STD_LOGIC_VECTOR(15 downto 0);     -- Masking Table
BLECF_A      : OUT STD_LOGIC_VECTOR(15 downto 0);     -- BLECF A Card ID
BLECF_B      : OUT STD_LOGIC_VECTOR(15 downto 0);     -- BLECF B Card ID
BLETC_SN_LSB : OUT STD_LOGIC_VECTOR(31 downto 0);     -- BLETC Serial Number (least significant bits)
BLETC_SN_MSB : OUT STD_LOGIC_VECTOR(31 downto 0);     -- BLETC Serial Number (most significant bits)
BLETC_FV     : OUT STD_LOGIC_VECTOR(31 downto 0);     -- BLETC_Firmware_version
MOD_DATE_LSB : OUT STD_LOGIC_VECTOR(31 downto 0);     -- Modification date of the LSA DB
MOD_DATE_MSB : OUT STD_LOGIC_VECTOR(31 downto 0);     -- Modification date of the LSA DB
ignoreCFC_A  : OUT STD_LOGIC;
ignoreCFC_B  : OUT STD_LOGIC
);

END Card_Settings;

-- architecture declaration

ARCHITECTURE RTL OF Card_Settings IS

   Signal
      Masking_int, 
      BIS_Connect_int, 
      Cable_Connect_int, 
      BLECF_A_int, 
      BLECF_B_int       : STD_LOGIC_VECTOR(15 downto 0); 

   Signal
      BLETC_SN_LSB_int,
      BLETC_SN_MSB_int,
      BLETC_FV_int,
      M_DATE_LSB_int,
      M_DATE_MSB_int    : STD_LOGIC_VECTOR(31 downto 0);

BEGIN

process (CLK, Reset)

   Variable Count_int   : STD_LOGIC_VECTOR(5 downto 0); 

BEGIN
   IF Reset = '1' THEN
      Masking_int       <= (others => '0');
      BIS_Connect_int   <= (others => '0');
      Cable_Connect_int <= (others => '0');
      BLECF_A_int       <= (others => '0');
      BLECF_B_int       <= (others => '0');
      BLETC_SN_LSB_int  <= (others => '0');
      BLETC_SN_MSB_int  <= (others => '0');
      BLETC_FV_int      <= (others => '0');
      M_DATE_LSB_int    <= (others => '0');
      M_DATE_MSB_int    <= (others => '0');
         
   ELSIF rising_edge(clk) THEN
      IF enable = '1' THEN 
          
         Count_int := Address;
          
         CASE Address IS 
            -- Reading the 16bit variables 
            WHEN "000010" => Cable_Connect_int  (15 DOWNTO  8) <= Data; --02
            WHEN "000011" => Cable_Connect_int  ( 7 DOWNTO  0) <= Data; --03
             
            WHEN "000110" => BIS_Connect_int    (15 DOWNTO  8) <= Data; --06
            WHEN "000111" => BIS_Connect_int    ( 7 DOWNTO  0) <= Data; --07
             
            WHEN "001010" => Masking_int        (15 DOWNTO  8) <= Data; --10
            WHEN "001011" => Masking_int        ( 7 DOWNTO  0) <= Data; --11
             
            -- Reserved space of one long (12 - 15) 
            
            WHEN "010010" => BLECF_A_int        (15 DOWNTO  8) <= Data; --18
            WHEN "010011" => BLECF_A_int        ( 7 DOWNTO  0) <= Data; --19
            
            WHEN "010110" => BLECF_B_int        (15 DOWNTO  8) <= Data; --22
            WHEN "010111" => BLECF_B_int        ( 7 DOWNTO  0) <= Data; --23
            
            -- Reading the 32bit variables
            WHEN "011000" => BLETC_SN_LSB_int   (31 DOWNTO 24) <= Data; --24
            WHEN "011001" => BLETC_SN_LSB_int   (23 DOWNTO 16) <= Data; --25
            WHEN "011010" => BLETC_SN_LSB_int   (15 DOWNTO  8) <= Data; --26
            WHEN "011011" => BLETC_SN_LSB_int   ( 7 DOWNTO  0) <= Data; --27
             
            WHEN "011100" => BLETC_SN_MSB_int   (31 DOWNTO 24) <= Data; --28
            WHEN "011101" => BLETC_SN_MSB_int   (23 DOWNTO 16) <= Data; --29
            WHEN "011110" => BLETC_SN_MSB_int   (15 DOWNTO  8) <= Data; --30
            WHEN "011111" => BLETC_SN_MSB_int   ( 7 DOWNTO  0) <= Data; --31
             
            WHEN "100000" => BLETC_FV_int       (31 DOWNTO 24) <= Data; --32
            WHEN "100001" => BLETC_FV_int       (23 DOWNTO 16) <= Data; --33
            WHEN "100010" => BLETC_FV_int       (15 DOWNTO  8) <= Data; --34
            WHEN "100011" => BLETC_FV_int       ( 7 DOWNTO  0) <= Data; --35
             
            -- Reserved space of five longs (36 - 55) 

            WHEN "111000" => M_DATE_LSB_int     (31 DOWNTO 24) <= Data; --56
            WHEN "111001" => M_DATE_LSB_int     (23 DOWNTO 16) <= Data; --57
            WHEN "111010" => M_DATE_LSB_int     (15 DOWNTO  8) <= Data; --58
            WHEN "111011" => M_DATE_LSB_int     ( 7 DOWNTO  0) <= Data; --59
             
            WHEN "111100" => M_DATE_MSB_int     (31 DOWNTO 24) <= Data; --60
            WHEN "111101" => M_DATE_MSB_int     (23 DOWNTO 16) <= Data; --61
            WHEN "111110" => M_DATE_MSB_int     (15 DOWNTO  8) <= Data; --62
            WHEN "111111" => M_DATE_MSB_int     ( 7 DOWNTO  0) <= Data; --63
            
            WHEN OTHERS =>
            
         END CASE;
      END IF; 
    
      Count          <= Count_int;
      Masking        <= Masking_int;
      BISConnect     <= BIS_Connect_int;
      CableConnect   <= Cable_Connect_int; 
      BLECF_A        <= BLECF_A_int; 
      BLECF_B        <= BLECF_B_int;
      BLETC_SN_LSB   <= BLETC_SN_LSB_int;
      BLETC_SN_MSB   <= BLETC_SN_MSB_int;
      BLETC_FV       <= BLETC_FV_int;
      MOD_DATE_LSB   <= M_DATE_LSB_int;   
      MOD_DATE_MSB   <= M_DATE_MSB_int;
      
      -- Checking for connected CFCs.
      
      ignoreCFC_A <= '0'; -- default value
      ignoreCFC_B <= '0'; -- default value
      
      if BLECF_A_int = X"0000" then ignoreCFC_A <= '1'; end if;
      if BLECF_B_int = X"0000" then ignoreCFC_B <= '1'; end if;
      
   END IF;

END PROCESS;

END RTL;


--- JUST Double checking addresses :) ---

--00   000000
--01   000001
------02   000010
------03   000011

--04   000100
--05   000101
------06   000110
------07   000111

--08   001000
--09   001001
------10   001010
------11   001011

--12   001100
--13   001101
--14   001110
--15   001111

--16   010000
--17   010001
------18   010010
------19   010011

--20   010100
--21   010101
------22   010110
------23   010111

------24   011000
------25   011001
------26   011010
------27   011011

------28   011100
------29   011101
------30   011110
------31   011111

------32   100000
------33   100001
------34   100010
------35   100011

--36   100100
--37   100101
--38   100110
--39   100111

--40   101000
--41   101001
--42   101010
--43   101011

--44   101100
--45   101101
--46   101110
--47   101111

--48   110000
--49   110001
--50   110010
--51   110011

--52   110100
--53   110101
--54   110110
--55   110111

------56   111000
------57   111001
------58   111010
------59   111011

------60   111100
------61   111101
------62   111110
------63   111111
