-- FUNCTION: DumpRequests.vhd
-- AUTHOR:   Christos Zamantzas
-- CREATED:  12 December 2009
-- VERSION:  12

-- Changes in ver. 2: Initial conversion to vhdl.
--                    Removed LPMs: Any_Dump, Connect_and, enable_counter
-- Changes in ver. 3: changed logic 
--                     - the ORed dump requests include the CS requests.
--                     - syncronisation happens before aclr and for all dump requests.
-- Changes in ver. 4: the DUMP_ON has its "cout" output inversed before is fed to the "en_clk" input.
-- Changes in ver. 5: registered the DumpReqA_int & DumpReqB_int signals.
-- Changes in ver. 6: clean-up and renaming of signals. No semantic changes.
-- Changes in ver. 7: changed logic 
--                     - synchronised the sorting of signals to maskable/unmaskable.
--                     - synchronised the removal of dumps from the unused channels.
-- Changes in ver. 8: added not toggling outputs for dumps.
-- Changes in ver. 9: added input from threshold update "TH_Update".
-- Changes in ver.10: added inputs from the RCC blocks.
-- Changes in ver.11: renamed dump output signals and inverted the DumpGlobal_U/M outputs.
-- Changes in ver.12: renamed signal Connection to Connect_BIS
--                    renamed signal Masking to Mask_BIS
--                    added a check to inhibit RCC dumps if channels from BLECF_A/B are not connected to BIS.


-- TODO: remove remaining LPM
-- TODO: add CableConnected flag and add it to the dump_inhibit logic.

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE IEEE.numeric_std.all;
USE IEEE.STD_LOGIC_UNSIGNED.all;

LIBRARY work;

ENTITY DumpRequests IS 
   PORT
   (
      Connect_BIS    : IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      Mask_BIS   : IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      Dump_A         : IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
      Dump_B         : IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
      RCC_dumpAB     : IN  STD_LOGIC;                       -- RCC Dump requests from BLECF A 
      RCC_dumpCD     : IN  STD_LOGIC;                       -- RCC Dump requests from BLECF B
      CSD_UNMASK     : IN  STD_LOGIC;                       -- Combiner Selected Dump for the UNMASKABLE line
      CSD_MASK       : IN  STD_LOGIC;                       -- Combiner Selected Dump for the MASKABLE line
      TH_Update      : IN  STD_LOGIC;                       -- Update of thresholds 
      clk            : IN  STD_LOGIC;
      
      UNMaskable     : OUT STD_LOGIC;                       -- toggling output 
      Maskable       : OUT STD_LOGIC;
      DumpReqA       : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);    -- detailed for BP_LOG
      DumpReqB       : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
      DumpGlobal_U   : OUT STD_LOGIC;                       -- non-toggling output; dump = '1'
      DumpGlobal_M   : OUT STD_LOGIC
   );
END DumpRequests;

ARCHITECTURE RTL OF DumpRequests IS 

COMPONENT dump_on
   PORT(
         clock  : IN STD_LOGIC;
         clk_en : IN STD_LOGIC;
         aclr   : IN STD_LOGIC;
         cout   : OUT STD_LOGIC;
         q      : OUT STD_LOGIC_VECTOR(20 DOWNTO 0)
       );
END COMPONENT;

SIGNAL Dump_ON_U_int  : STD_LOGIC;
SIGNAL Dump_ON_M_int  : STD_LOGIC;
SIGNAL Dump_UNMASK    : STD_LOGIC;
SIGNAL Dump_MASK      : STD_LOGIC;
SIGNAL RCC_AB_int     : STD_LOGIC;
SIGNAL RCC_CD_int     : STD_LOGIC;

SIGNAL DumpReqA_int   : STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL DumpReqB_int   : STD_LOGIC_VECTOR(7 DOWNTO 0);

SIGNAL Mask_BISN       : STD_LOGIC_VECTOR(15 DOWNTO 0);

SIGNAL UNMaskable_out : STD_LOGIC_VECTOR(3 DOWNTO 0);
SIGNAL Maskable_out   : STD_LOGIC_VECTOR(3 DOWNTO 0);

SIGNAL UNMASK         : STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL MASK           : STD_LOGIC_VECTOR(15 DOWNTO 0);

BEGIN 

-- instantiate counters: 
--    max bits of counter define the time the beam permit line will be hold down.
 
inst1 : dump_on
PORT MAP(
       clock  => clk,
       clk_en => NOT(Dump_ON_M_int),
       aclr   => Dump_MASK,
       cout   => Dump_ON_M_int
       );

inst2 : dump_on
PORT MAP(
       clock  => clk,
       clk_en => NOT(Dump_ON_U_int),
       aclr   => Dump_UNMASK ,
       cout   => Dump_ON_U_int
       );

PROCESS(clk)
BEGIN
   IF (RISING_EDGE(clk)) THEN
      
      RCC_AB_int <= RCC_dumpAB;
      -- Inhibit RCC dumps from BLECF A if its channels are not connected to BIS
      if Connect_BIS(7 DOWNTO 0) = X"00" then 
         RCC_AB_int <= '0'; 
      end if;
      
      RCC_CD_int <= RCC_dumpCD;
      -- Inhibit RCC dumps from BLECF B if its channels are not connected to BIS
      if Connect_BIS(15 DOWNTO 8) = X"00" then 
         RCC_CD_int <= '0'; 
      end if;
    
      Mask_BISN <= NOT(Mask_BIS);
      
      -- Removing dump requests from not connected channels
      DumpReqA_int <= Connect_BIS(7 DOWNTO 0)  AND Dump_A;
      DumpReqB_int <= Connect_BIS(15 DOWNTO 8) AND Dump_B;
      
      -- Sorting dump requests to MASKABLE and UNMASKABLE
      UNMASK(7 DOWNTO 0)  <= Mask_BISN(7 DOWNTO 0)  AND DumpReqA_int;
      UNMASK(15 DOWNTO 8) <= Mask_BISN(15 DOWNTO 8) AND DumpReqB_int;
       
      MASK(7 DOWNTO 0)    <= Mask_BIS(7 DOWNTO 0)   AND DumpReqA_int;
      MASK(15 DOWNTO 8)   <= Mask_BIS(15 DOWNTO 8)  AND DumpReqB_int;
      
      -- Generating the output signals
      Dump_UNMASK  <= 
                      TH_Update  OR
                      RCC_AB_int OR
                      RCC_CD_int OR
                      CSD_UNMASK OR 
                      
                      UNMASK(15) OR 
                      UNMASK(14) OR 
                      UNMASK(13) OR 
                      UNMASK(12) OR 
                      UNMASK(11) OR 
                      UNMASK(10) OR 
                      UNMASK(9)  OR 
                      UNMASK(8)  OR 
                      UNMASK(7)  OR 
                      UNMASK(6)  OR 
                      UNMASK(5)  OR 
                      UNMASK(4)  OR 
                      UNMASK(3)  OR 
                      UNMASK(2)  OR 
                      UNMASK(1)  OR 
                      UNMASK(0);
      
      Dump_MASK    <= 
                      TH_Update  OR
                      RCC_AB_int OR
                      RCC_CD_int OR
                      CSD_MASK   OR 
                      
                      MASK(15)   OR 
                      MASK(14)   OR 
                      MASK(13)   OR 
                      MASK(12)   OR 
                      MASK(11)   OR 
                      MASK(10)   OR 
                      MASK(9)    OR 
                      MASK(8)    OR 
                      MASK(7)    OR 
                      MASK(6)    OR 
                      MASK(5)    OR 
                      MASK(4)    OR 
                      MASK(3)    OR 
                      MASK(2)    OR 
                      MASK(1)    OR 
                      MASK(0);
      
      -- count-up unless dump request (UNMASKABLE).
      UNMaskable_out <= UNMaskable_out + '1';
      IF Dump_ON_U_int = '0' THEN 
         UNMaskable_out <= (others => '0');
      END IF;
      
      -- count-up unless dump request (MASKABLE).
      Maskable_out <= Maskable_out + '1';
      IF Dump_ON_M_int = '0' THEN 
         Maskable_out <= (others => '0');
      END IF;
      
   END IF;
END PROCESS;

UNMaskable     <= UNMaskable_out(3); -- use the MSB of the counter to create the toggling output (UNMASKABLE).
Maskable       <= Maskable_out(3);   -- use the MSB of the counter to create the toggling output (MASKABLE).
DumpReqA       <= DumpReqA_int;
DumpReqB       <= DumpReqB_int;
DumpGlobal_U   <= NOT Dump_ON_U_int;
DumpGlobal_M   <= NOT Dump_ON_M_int;


END RTL;