-- FILE: BP_LOG_CTRL.vhd (was: BeamPermit_CTRL.vhd)
-- DESCRIPTION: BEAM PERMIT LOGGING CONTROL
--              Three-State Moore State Machine  

-- AUTHOR: Christos Zamantzas
   
-- VERSION: 17
-- DATE: 15/11/2009

-- changes in ver.07: added read_last_addr signal to notify when last address has been read.
-- changes in ver.08: added one cycle delay in the reading of the memory data.
-- changes in ver.09: merged reading and writing; added two cycles delay in the reading of the memory data.
-- changes in ver.10: removed the "temp_addr" signal (wasn't being used).
-- changes in ver.11: rename of signals. no semantic changes.
-- changes in ver.12: modified reset to be synchronous.
-- changes in ver.13: moved the clearContents condition to externally triggered.
-- changes in ver.14: changing controls since moving to a true dual-port memory (proposed by Csaba).
--                    renaming of signals for readability.
-- changes in ver.15: changing LOG_addr to use unsigned. (proposed by Csaba).
--                    renaming file from BeamPermit_CTRL to BP_LOG_CTRL (ATTN: in SVN old files with the same name exists!)
--                    added some more comments.
-- changes in ver.16: semantic correction of recording state.
--                    changed WordCnt to NATURAL for consistency with ver. 15(proposed by Csaba).
-- changes in ver.17: removed unecessary signal read_last_addr since it is not used anymore.

-- stats for the complete block BP_LOG:
--      1 x M512  for the FIFO, 
--      1 x M4K   for the memory, 
--    100 x LE    here!. 

library ieee;
use ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

entity BP_LOG_CTRL is

   port(
      data_from_log : IN  STD_LOGIC_VECTOR(7 downto 0);
      FIFO_data     : IN  STD_LOGIC_VECTOR(7 downto 0);
      FIFO_addr     : IN  STD_LOGIC_VECTOR(3 downto 0);
      FIFO_empty    : IN  STD_LOGIC;
      clr_log       : IN  STD_LOGIC;
      clk           : IN  STD_LOGIC;
      reset         : IN  STD_LOGIC;
      
      data_to_log   : OUT STD_LOGIC_VECTOR(7 downto 0);
      LOG_addr      : OUT STD_LOGIC_VECTOR(3 downto 0);
      LOG_wren      : OUT STD_LOGIC;
      FIFO_rdreq    : OUT STD_LOGIC   
   );

end entity;

architecture rtl of BP_LOG_CTRL is

   -- Build an enumerated type for the state machine;
   type state_type is (idle, CleanMemoryContents, RecordingData);

   -- Define Register to hold the current state;
   signal state            :state_type;
   -- Define Registers to hold data or flags;
   signal read_locked      :std_logic;
   signal read_ended       :std_logic;
   signal FIFO_tmp         :std_logic_vector(7 downto 0);
   signal clr_log_int      :std_logic; 

   signal WordCnt          :natural range 0 to 16;
   
begin

   -- Define Variables;
   
   -- Logic to advance to the next state;
   process (clk, reset)
   begin
      
      if (rising_edge(clk)) then
         
         if reset = '1' then
            LOG_wren       <= '0'; 
            FIFO_rdreq     <= '0'; 
            WordCnt        <= 0;
            state          <= CleanMemoryContents;
          
         else
           
            if clr_log = '1' then       -- clr signal needs to be latched!
              clr_log_int <= '1'; 
            end if;
           
            LOG_wren      <= '0';       -- dont write if not needed
            
            case state is
             
               when idle => 
                  state <= idle;
                   
                  if (FIFO_empty = '0') then
                     state       <= RecordingData;
                     WordCnt     <= 0;       -- clear counter 
                     FIFO_rdreq  <= '1';
                   
                  elsif (clr_log_int = '1') then
                     state       <= CleanMemoryContents;
                     clr_log_int <= '0';     -- clear latch.
                     
                  end if;
                
               when CleanMemoryContents =>
                
                  data_to_log <= (others => '0');
                  LOG_addr    <= std_logic_vector(to_unsigned((WordCnt),4));
                  LOG_wren    <= '1';
                  WordCnt     <= WordCnt + 1; -- increment counter
                     
                  state       <= CleanMemoryContents; 
                     
                  if ( WordCnt = 15 ) then    -- end of cleaning
                     state <= idle;
                  end if;
                
               when RecordingData =>

                  state <= RecordingData;                 -- remain here till log is created.
                
                  case WordCnt is
                   
                     when 0 =>
                        FIFO_rdreq   <= '0';              -- release flag; new data are now available
                        FIFO_tmp     <= FIFO_data;        -- store data to use later for writting
                        LOG_addr     <= FIFO_addr;        -- select read/write address
                     when 1 =>                            -- do nothing; wait for mem data (1st cycle)
                     when 2 =>                            -- do nothing; wait for mem data (2nd cycle)
                     when 3 =>
                        LOG_wren     <= '1';
                        data_to_log  <= FIFO_tmp or data_from_log; -- adding new dumps to the already existing.
                        state        <= idle;
                     when others =>
                        state        <= idle;
                   
                  end case;
                   
                  WordCnt <= WordCnt + 1;                 -- increment counter
                
               when others =>
                
                  state <= idle;
             
            end case;
            
         end if;
      end if;
   end process;

end rtl;
