----------------------------------------------------------------------
-- Unit name: ReceiverEnergy
----------------------------------------------------------------------
-- File name: ReceiverEnergy.vhdl
----------------------------------------------------------------------
-- Designed by: jonathan Emery (AB/BI/BL)
-- Modified by: 
----------------------------------------------------------------------
-- Date: 26.08.2008
----------------------------------------------------------------------
-- Description:
-- Transmission byte oriented, Most significant Byte first, Less significant bit first.
-- with parity and bit sycro for each byte / Manchester encoding (as per IEEE 802.3)

-- Structure:
-- Preamble / synchro / byte3 / parity / sychro / byte2 / parity / synchro / byte1 / parity / synchro / byte0 / parity / queue

-- Preamble: RZ encoding, "0101010101000111" (MSbit first)
-- Synchro:  Manchester encoding, '1'
-- Byte:     Manchester encoding (LSbit first)
-- Parity:   Parity bit											ignore in this version
-- Queue:    RZ encoding, "101111" (MSbit first)

-- 4x8 + 4x2  32+8  = 40 bits of data

-- 4 bytes data => 32 bits
-- Organization:
-- Eframe <= "100" (Energy frame) & "1" (LHC Accelerator) & "000" & Toggle_bit & Energy(15 downto 0) & CRC(7 downto 0); 
----------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.Numeric_std.all;


ENTITY ReceiverEnergy IS
PORT
	(
			Reset 					: in STD_LOGIC;
			Clk 					: in STD_LOGIC;
			SDataIn					: in STD_LOGIC;			
			DataValid				: out STD_LOGIC;
			CRCcorrect				: out STD_LOGIC;
			ToggleBit				: out STD_LOGIC;
			EnergyOut				: out STD_LOGIC_VECTOR(15 downto 0)		
	);
END ReceiverEnergy;

architecture basic of ReceiverEnergy is

	signal SDATA_OLD	 	: Std_logic;
	signal COUNT_INSIDE_BIT : UNSIGNED(5 downto 0);
	
	signal SB0, SB1, SB2	: Std_logic;
	signal NEW_BIT			: Std_logic;
	signal BIT_SAMPLED		: Std_logic;
	
	signal DETECT_SOF		: Std_logic_vector(5 downto 0);  --Mem the SDataIn to detect pattern of start of frame
	signal SOF, IN_FRAME	: Std_logic;
	
	signal COUNT_BIT		: UNSIGNED(5 downto 0);
	
	signal BIT0				: Std_logic;
	signal FIRST_BIT		: Std_logic;
	signal DATA_PARALLEL	: Std_logic_vector(39 downto 0);
	
	signal FRAME_IS_ENERGY	: Std_logic;
	signal C				: Std_logic_vector(7 downto 0);
	signal D				: Std_logic_vector(23 downto 0);
	signal NewCRC			: Std_logic_vector(7 downto 0);
	
	
begin -- basic architecture

GenerateCounterSynchroWithBit : process(Reset, Clk, SDataIn)
begin
	if Reset = '1' then
		SDATA_OLD <= '0';
		COUNT_INSIDE_BIT <= "000000";
	elsif Rising_edge(Clk) then
		if (SDATA_OLD xor SDataIn) = '1' then  -- detect rising and falling edge 
			COUNT_INSIDE_BIT <= "000000";
		else
			COUNT_INSIDE_BIT <= COUNT_INSIDE_BIT + 1;
			if COUNT_INSIDE_BIT = 40 then
				COUNT_INSIDE_BIT <= "000000";
			end if;
		end if;
		SDATA_OLD <= SDataIn;
	end if;
end process;

OverSampleBit: process(Reset, Clk, COUNT_INSIDE_BIT, SDataIn)			-- Over sample 3 times the bit, create BIT_SAMPLED
begin
	if Reset = '1' then
		BIT_SAMPLED <= '0';
	elsif Rising_edge(Clk) then
		if COUNT_INSIDE_BIT = 8 then
			SB0  <= SDataIn;
		elsif COUNT_INSIDE_BIT = 16 then
			SB1  <= SDataIn;
		elsif COUNT_INSIDE_BIT = 32 then
			SB2  <= SDataIn;
		elsif COUNT_INSIDE_BIT = 33 then
			if ((SB0 and SB1) or (SB1 and SB2) or (SB0 and SB2)) = '1' then
				BIT_SAMPLED <= '1';
			else 
				BIT_SAMPLED <= '0';
			end if;
			NEW_BIT <= '1';
		else 
			NEW_BIT <= '0';
		end if;
	end if;
end process;
	
DetectSOF : process(Reset, Clk, NEW_BIT, BIT_SAMPLED)  -- USE the the end of the preambule with the 111
begin
	if Reset = '1' then
		SOF <= '0';
		DETECT_SOF <= "000000";
	elsif Rising_edge(Clk) then
		
		if NEW_BIT = '1' then
			DETECT_SOF <= DETECT_SOF(4 downto 0) & BIT_SAMPLED;
		end if;
		
		if DETECT_SOF = "000111" then
			SOF <= '1';
		else
			SOF <= '0';
		end if;
		
	end if;	
end process;

ShiftData : process(Reset, Clk, SOF, NEW_BIT, BIT_SAMPLED, COUNT_BIT, IN_FRAME)  -- 
begin
	if Reset = '1' then
		BIT0  <= '0';
		COUNT_BIT <= "000000";
		IN_FRAME <= '0';
		FIRST_BIT <= '1';
	elsif Rising_edge(Clk) then 
	  if NEW_BIT = '1' then
				
		if SOF = '1' then
		  IN_FRAME <= '1';
		end if;
				
		if SOF = '1' or IN_FRAME = '1' then
			
			if FIRST_BIT = '1' then	
				BIT0 <= BIT_SAMPLED; 											-- Save new bit 0
				FIRST_BIT <= '0';
			else
				FIRST_BIT <= '1';
				
				if BIT0 = '0' and BIT_SAMPLED = '1' then  -- '1'				--process the previous data
					DATA_PARALLEL <= '1' & DATA_PARALLEL(39 downto 1);
				elsif BIT0 = '1' and BIT_SAMPLED = '0' then -- '0'
					DATA_PARALLEL <= '0' & DATA_PARALLEL(39 downto 1);
				--else 
					-- wrong state...... will be picked up by the CRC check
				end if;
							
				if COUNT_BIT < 40 then
					COUNT_BIT <= COUNT_BIT + 1;
				end if;
			end if;
			
		end if;
		
		-- When bit count = 40 the last bit has already been completed
		if COUNT_BIT = 40 then								-- go only ones inside this part
			EnergyOut <= DATA_PARALLEL(18 downto 11) & DATA_PARALLEL(28 downto 21);
			ToggleBit <= DATA_PARALLEL(1);
			IN_FRAME <= '0'; 
			COUNT_BIT <= "000000";
			FIRST_BIT <= '1';
		end if;	
		
	  end if;			
	end if;	
end process;

CheckCRC_Header: process(Reset, Clk, DATA_PARALLEL, COUNT_BIT, C, D, NEW_BIT)
begin
	C <= "00000000";
	D <= DATA_PARALLEL(8 downto 1) & DATA_PARALLEL(18 downto 11) & DATA_PARALLEL(28 downto 21);
	
	NewCRC(0) <= D(23) xor D(21) xor D(19) xor D(18) xor D(16) xor D(14) xor 
                 D(12) xor D(8) xor D(7) xor D(6) xor D(0) xor C(0) xor 
                 C(2) xor C(3) xor C(5) xor C(7);
	NewCRC(1) <= D(23) xor D(22) xor D(21) xor D(20) xor D(18) xor D(17) xor 
                 D(16) xor D(15) xor D(14) xor D(13) xor D(12) xor D(9) xor 
                 D(6) xor D(1) xor D(0) xor C(0) xor C(1) xor C(2) xor 
                 C(4) xor C(5) xor C(6) xor C(7);
	NewCRC(2) <= D(22) xor D(17) xor D(15) xor D(13) xor D(12) xor D(10) xor 
                 D(8) xor D(6) xor D(2) xor D(1) xor D(0) xor C(1) xor 
                 C(6);
	NewCRC(3) <= D(23) xor D(18) xor D(16) xor D(14) xor D(13) xor D(11) xor 
                 D(9) xor D(7) xor D(3) xor D(2) xor D(1) xor C(0) xor 
                 C(2) xor C(7);
	NewCRC(4) <= D(19) xor D(17) xor D(15) xor D(14) xor D(12) xor D(10) xor 
                 D(8) xor D(4) xor D(3) xor D(2) xor C(1) xor C(3);
	NewCRC(5) <= D(20) xor D(18) xor D(16) xor D(15) xor D(13) xor D(11) xor 
                 D(9) xor D(5) xor D(4) xor D(3) xor C(0) xor C(2) xor 
                 C(4);
	NewCRC(6) <= D(21) xor D(19) xor D(17) xor D(16) xor D(14) xor D(12) xor 
                 D(10) xor D(6) xor D(5) xor D(4) xor C(0) xor C(1) xor 
                 C(3) xor C(5);
	NewCRC(7) <= D(22) xor D(20) xor D(18) xor D(17) xor D(15) xor D(13) xor 
                 D(11) xor D(7) xor D(6) xor D(5) xor C(1) xor C(2) xor 
                 C(4) xor C(6);
                                  
	if Reset = '1' then 
		CRCcorrect <= '0';
		DataValid  <= '0';
	elsif Rising_edge(Clk) then 
		DataValid  <= '0';
		CRCcorrect <= '0';
		if COUNT_BIT = 40 and NEW_BIT = '1' then
			if NewCRC = DATA_PARALLEL(38 downto 31) then
				CRCcorrect <= '1';
			end if;
			if DATA_PARALLEL(8 downto 5) = "1001" then  -- This is a energy frame
				DataValid <= '1'; 
			end if;
		end if;
	end if;
	
end process;

end architecture basic;