-- File: SRAM_Control.vhd
-- Description: SRAM control for the PM and BD buffers.
--                There are 6 x 32 bit words to be read from each set of 8 monitors.
--                This trivial FSM goes through when data available from both CFCs in the fifo and reads those 12 values 
--                  by selecting the correct input from the MUX and acknowledges when data have been read from the fifo.
--                This block is also responsible to start/stop the writing and the address of the SRAM.
--                Later additions provide protection from different data rates between CFC modules by producing half packets,
--                  i.e. recording data from the faster CFC and padding the missing data.
-- Warnings:      Versions 13 to 18 should not be used in production enviroments
--                Versions >= 19 requires handling of the "half-packets" by post-processing.
--
-- Author: Christos Zamantzas
-- Version: 21
-- Date: 12/08/2010

-- 16/04/2007 ver.09: this version is trimmed down to perform only major functions.
-- 04/12/2009 ver.10: beatification of code.
--                    added a force stop for the address counting when reset
-- 29/01/2010 ver.11: added more signals in the reset routine;
--                    fifo_CHECK state has been deprecated and function slited in the idle and start states.
--                    added a function completed state. 
--                    changed logic - after read/write of data returns to idle state instead of checking for new data immediatelly.
--                    added some more comments.
-- 21/02/2010 ver.12: correction of semantic error - the address counter was left counting one cycle too much.
-- 20/07/2010 ver.13: rename of fifo flags to more appropriate names.
-- 21/07/2010 ver.14: added one clock cycle delay to match the synchronous MUX in the higher level controled by the "Sel_Data".
--                    changed the word order to match change in the MUX (from 6 words to 8).
-- 22/07/2010 ver.15: corrected a semantic error with the initiation of writing to the memory
--                    removed one state on the loop writting the data.
--                    added housekeeping function in the IDLE state
-- 23/07/2010 ver.16: importing here the handling of all the fifos flags and adding the full flag
--                    adding logic to handle different data rates of the CFC links.
-- 23/07/2010 ver.17: changed the sequence of the full fifo checks.
--                    clean-up of code.
-- 24/07/2010 ver.18: changed the RdAck to ReReq to match the change of fifo type (for better performance).
--                    changed the logic to request new data from the fifo (instead of acknowledge its read).
-- 24/07/2010 ver.19: correction of semantic error - data should be requested from fifo one cycle ahead.
-- 02/08/2010 ver.20: added some more safety in the part of code checking which path to be taken in the state machine.
-- 12/08/2010 ver.21: correction of semantic error - data requested from fifo B was kept for more cycles than needed.
--                    correction of semantic error - data was never requested from fifo B if previous state was empty half packet A.

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.Numeric_std.all;

ENTITY SRAM_Control IS
   PORT
   (
      enable       : IN  STD_LOGIC;
      fifo1_full   : IN  STD_LOGIC;
      fifo1_ready  : IN  STD_LOGIC;
      fifo1_empty  : IN  STD_LOGIC;
      fifo2_full   : IN  STD_LOGIC;
      fifo2_ready  : IN  STD_LOGIC;
      fifo2_empty  : IN  STD_LOGIC;
      reset        : IN  STD_LOGIC;
      clk          : IN  STD_LOGIC;
      SRAM_wrtN    : OUT STD_LOGIC;
      inc_Counter  : OUT STD_LOGIC;
      fifo_RdReq1x : OUT STD_LOGIC;
      fifo_RdReq2x : OUT STD_LOGIC;
      Sel_Data     : OUT STD_LOGIC_VECTOR (2 DOWNTO 0)
   );
   
END SRAM_Control;

ARCHITECTURE RTL OF SRAM_Control IS
   TYPE STATE_TYPE IS 
   (
      IDLE,
      READ_START,
      READ_A1,
      READ_A2,
      READ_A3,
      READ_A4,
      READ_A5,
      READ_A6,
      READ_B1,
      READ_B2,
      READ_B3,
      READ_B4,
      READ_B5,
      READ_B6,
      READ_A1_empty,
      READ_A2_empty,
      READ_A3_empty,
      READ_A4_empty,
      READ_A5_empty,
      READ_A6_empty,
      READ_B1_empty,
      READ_B2_empty,
      READ_B3_empty,
      READ_B4_empty,
      READ_B5_empty,
      READ_B6_empty,
      READ_COMPLETED
   );

   SIGNAL state          : STATE_TYPE;
   SIGNAL WriteN_int     : STD_LOGIC;
   SIGNAL fifo_ready     : STD_LOGIC;
   SIGNAL fifo_empty     : STD_LOGIC;
   SIGNAL fifo1_full_int : STD_LOGIC;
   SIGNAL fifo2_full_int : STD_LOGIC;
   SIGNAL emptyBpacket   : STD_LOGIC;

BEGIN
   PROCESS (clk, reset)
   BEGIN
      IF reset = '1' THEN
         state <= IDLE;
         inc_Counter  <= '0';   -- stop incrementing address.
         fifo_RdReq1x <= '0';   -- stop clearing the fifos
         fifo_RdReq2x <= '0';
         SRAM_wrtN    <= '1';   -- stop writing data to SRAM
         
      ELSIF clk'EVENT AND clk = '1' THEN
      
         -- Preparation of flags
         fifo_ready <= fifo1_ready and fifo2_ready;
         fifo_empty <= fifo1_empty or fifo2_empty;
         fifo1_full_int <= fifo1_full;
         fifo2_full_int <= fifo2_full;
         
         CASE state IS
          
            WHEN IDLE =>
               state <= IDLE; -- Stay here if no new data. 
               
               -- Housekeeping (just in case the return happens from the "other" state)
               fifo_RdReq1x <= '0';   -- stop clearing the fifo
               fifo_RdReq2x <= '0';   -- stop clearing the fifo
               SRAM_wrtN    <= '1';   -- stop writing data to SRAM
               inc_Counter  <= '0';   -- stop incrementing the address
               emptyBpacket <= '0';   -- clear flag 
               
               IF enable = '1' THEN
                 Sel_Data <= "000";  -- Choose first word
                 
                 -- 1st fifo is full; produce "half" packet 
                 IF fifo1_full_int = '1' and fifo2_full_int = '0' THEN
                   state  <= READ_START;
                   fifo_RdReq1x <= '1'; -- Read A next - pulse start
                   emptyBpacket <= '1';
                 -- 2nd fifo is full; produce "half" packet
                 ELSIF fifo1_full_int = '0' and fifo2_full_int = '1' THEN
                   state  <= READ_A1_empty;
                   Sel_Data <= "011";
                 -- normal operation; produce "full" packet
                 ELSIF fifo_ready = '1' THEN
                   state <= READ_START;
                   fifo_RdReq1x <= '1'; -- Read A next - pulse start
                 END IF;
               END IF;
            
            WHEN READ_START =>      -- just waiting 1 cycle for data to be available from fifo
               fifo_RdReq1x <= '0'; -- Read A next - pulse end
               state <= READ_A1;
               
            -- PACKET MODE: "A part is available"
            WHEN READ_A1 =>
               SRAM_wrtN    <= '0'; -- Start writing to SRAM   
               inc_Counter  <= '1'; -- Start incrementing the address 
               Sel_Data <= "001";
               state <= READ_A2;
            WHEN READ_A2 =>
               fifo_RdReq1x <= '1'; -- Read A next - pulse start
               Sel_Data <= "010";
               state <= READ_A3;
            WHEN READ_A3 =>
               fifo_RdReq1x <= '0'; -- Read A next - pulse end
               Sel_Data <= "000";
               state <= READ_A4;
            WHEN READ_A4 =>
               Sel_Data <= "001";
               state <= READ_A5;
            WHEN READ_A5 =>
               Sel_Data <= "010";
               state <= READ_A6;
               -- requesting data from fifo2 if in "full" packet mode.
               IF emptyBpacket = '0' THEN
                 fifo_RdReq2x <= '1'; -- Read B next - pulse start
               END IF;
            WHEN READ_A6 =>
               -- calculating next state based on packet mode.
               IF emptyBpacket = '1' THEN
                 emptyBpacket <= '0';
                 Sel_Data <= "111";
                 state <= READ_B1_empty;
               ELSE
                 fifo_RdReq2x <= '0'; -- Read B next - pulse end
                 Sel_Data <= "100";
                 state <= READ_B1;
               END IF;
            
            -- PACKET MODE: "half-packet with missing A part"
            WHEN READ_A1_empty =>
               SRAM_wrtN   <= '0'; -- Start writing to SRAM   
               inc_Counter <= '1'; -- Start incrementing the address 
               state <= READ_A2_empty;
            WHEN READ_A2_empty =>
               state <= READ_A3_empty;
            WHEN READ_A3_empty =>
               state <= READ_A4_empty;
            WHEN READ_A4_empty =>
               state <= READ_A5_empty;
            WHEN READ_A5_empty =>
               fifo_RdReq2x <= '1'; -- Read B next - pulse start
               state <= READ_A6_empty;
            WHEN READ_A6_empty =>
               fifo_RdReq2x <= '0'; -- Read B next - pulse end
               Sel_Data <= "100";
               state <= READ_B1;
               
            -- PACKET MODE: "B part is available"   
            WHEN READ_B1 =>
               Sel_Data <= "101";
               state <= READ_B2;
            WHEN READ_B2 =>
               fifo_RdReq2x <= '1'; -- Read B next - pulse start
               Sel_Data <= "110";
               state <= READ_B3;
            WHEN READ_B3 =>
               fifo_RdReq2x <= '0'; -- Read B next - pulse end
               Sel_Data <= "100";
               state <= READ_B4;
            WHEN READ_B4 =>
               Sel_Data <= "101";
               state <= READ_B5;
            WHEN READ_B5 =>
               Sel_Data <= "110";
               state <= READ_B6;
            WHEN READ_B6 =>
               state <= READ_COMPLETED;
               
            -- PACKET MODE: "half-packet with missing B part"
            WHEN READ_B1_empty =>
               state <= READ_B2_empty;
            WHEN READ_B2_empty =>
               state <= READ_B3_empty;
            WHEN READ_B3_empty =>
               state <= READ_B4_empty;
            WHEN READ_B4_empty =>
               state <= READ_B5_empty;
            WHEN READ_B5_empty =>
               state <= READ_B6_empty;
            WHEN READ_B6_empty =>
               state <= READ_COMPLETED;
               
            -- Operation completed
            WHEN READ_COMPLETED =>
               inc_Counter  <= '0'; -- Stop incrementing the address
               SRAM_wrtN    <= '1'; -- Stop writing to SRAM
               state <= IDLE; 
            
            -- Safe State Machine
            WHEN OTHERS =>
               state <= IDLE;
          
         END CASE;
      END IF;
   END PROCESS;
END RTL;
