-- FILE        : ERR.vhd
-- DESCRIPTION : Chooses the correct optical link, generates ERR and dump signals
-- DESIGNER    : Christos Zamantzas
-- VERSION     : 12
-- DATE        : 2010/04/01

-- Changes in ver. 02: Initial conversion to vhdl.
-- Changes in ver. 03: clocked/synchronised function.
--                     complete change of logic using case statements
--                     added "link is down" signals in the logic
-- Changes in ver. 04: added "Ready_Chx" inputs and "EnableRead" output.
-- Changes in ver. 05: added ignoreLKA & ignoreLKB flags to inhibit dumps from statuses.
-- Changes in ver. 06: moved EnableRead inside the process.
-- Changes in ver. 07: new truth table with more dumps 
--                     ATTN: last case is not implemented 
--                       - need to find a way to ignore first the empty slots for BLECFs
-- Changes in ver. 08: adding missing cases from the truth table
--                     added ignoreCFC signal to mask unecessary dumps
--                     removed ignoreLKA/B outputs
-- Changes in ver. 09 [Csaba F. Hajdu]:
--                     merger of ERR.vhd and the VHDL version of CheckStatuses
--                     Read enable signal chain moved in from RCC.vhd
--                     ERRA and ERRB outputs of RCC block driven from this block
--                     ignoreCFC doesn't inhibit error counts but directly inhibits DUMP output
--                     Bugfixes: ERRD and ERRF inhibited for links with errors/links that are down (see specification)
-- Changes in ver. 10 [Csaba F. Hajdu]:
--                     ERRA and ERRB: single output pulse is generated in ERR.vhd
-- Changes in ver. 11 [Csaba F. Hajdu]: 
--                     Current vs. previous FID: change in specification! Added truth tables for checks
--                       Checks no longer done in-channel, "correct" previous FID is checked vs. "correct" current FID
-- Changes in ver. 12 [Csaba F. Hajdu]: 
--                     ERRA_D, ERRB_D, LK_A_D, LK_B_D no longer cleared at reset: logic too complicated, short FID dump at startup can be lived with




-- TRUTH TABLE for DUMPS BASED ON STATUSES:
-------------------------------------------------------------------------------------------------------------------------------------------------------
--| LK_A | LK_B | ERRA | ERRB | ERRC | CHOICE | Comment                                 | P | Cases                                                   |
--|------|------|------|------|------|--------|-----------------------------------------|---|----------------------------------------------------------
--|  0   |  0   |   0  |  0   |  0   |  LK_B  | all ok - choice based on PCB properties | 1 | 00000                                                   |
--|  0   |  0   |   0  |  0   |  1   |  LK_B  | CID or FID checks should give this dump | 1 | 00001                                                   |
--|  0   |  0   |   0  |  1   |  x   |  LK_A  | LK_B has errors                         | 2 | 00010, 00011                                            |
--|  0   |  0   |   1  |  0   |  x   |  LK_B  | LK_A has errors                         | 2 | 00100, 00101                                            |
--|  0   |  0   |   1  |  1   |  x   |  Dump  | LK_A and LK_B have errors               | 2 | 00110, 00111                                            |
--|  0   |  1   |   0  |  x   |  x   |  LK_A  | LK_B is down                            | 4 | 01000, 01001, 01010, 01011                              |
--|  0   |  1   |   1  |  x   |  x   |  Dump  | LK_B is down and LK_A has error         | 4 | 01100, 01101, 01110, 01111                              |
--|  1   |  0   |   x  |  0   |  x   |  LK_B  | LK_A is down                            | 4 | 10000, 10001, 10100, 10101                              |
--|  1   |  0   |   x  |  1   |  x   |  Dump  | LK_A is down and LK_B has error         | 4 | 10010, 10011, 10110, 10111                              |
--|  1   |  1   |   x  |  x   |  x   |  Dump  | LK_A and LK_B are both down             | 8 | 11000, 11001, 11010, 11011, 11100, 11101, 11110, 11111  |
-------------------------------------------------------------------------------------------------------------------------------------------------------
-- where '0' = OK, '1' = ERROR, x = don't care.


-- TRUTH TABLE for DUMPS BASED ON ChA vs ChB CID COMPARISONS:
--------------------------------------------------------------------------------------------------------------------------------------------------
--| CIDa /= CIDb | LK_Ac | LK_Bc | ERRAc | ERRBc || ERRD  | DUMP  | Comment                                                                      |
--|--------------|-------|-------|-------|-------||-------|--------------------------------------------------------------------------------------|
--|      0       |   x   |   x   |   x   |   x   ||   0   | no    | No CID comparison error                                                      |
--|      1       |   1   |   x   |   x   |   x   ||   0   | no    | CID comparison error - dump inhibited by link A being down                   |
--|      1       |   x   |   1   |   x   |   x   ||   0   | no    | CID comparison error - dump inhibited by link B being down                   |
--|      1       |   x   |   x   |   1   |   x   ||   0   | no    | CID comparison error - dump inhibited by link A having errors                |
--|      1       |   x   |   x   |   x   |   1   ||   0   | no    | CID comparison error - dump inhibited by link B having errors                |
--|      1       |   0   |   0   |   0   |   0   ||   1   | DUMP  | CID comparison error - DUMP                                                  |
--------------------------------------------------------------------------------------------------------------------------------------------------
-- where '0' = OK, '1' = ERROR, x = don't care.
--       c: current.


-- TRUTH TABLE for DUMPS BASED ON ChA vs ChB FID COMPARISONS:
--------------------------------------------------------------------------------------------------------------------------------------------------
--| FIDa /= FIDb | LK_Ac | LK_Bc | ERRAc | ERRBc || ERRF  | DUMP  | Comment                                                                      |
--|--------------|-------|-------|-------|-------||-------|--------------------------------------------------------------------------------------|
--|      0       |   x   |   x   |   x   |   x   ||   0   | no    | No FID comparison error                                                      |
--|      1       |   1   |   x   |   x   |   x   ||   0   | no    | FID comparison error - dump inhibited by link A being down                   |
--|      1       |   x   |   1   |   x   |   x   ||   0   | no    | FID comparison error - dump inhibited by link B being down                   |
--|      1       |   x   |   x   |   1   |   x   ||   0   | no    | FID comparison error - dump inhibited by link A having errors                |
--|      1       |   x   |   x   |   x   |   1   ||   0   | no    | FID comparison error - dump inhibited by link B having errors                |
--|      1       |   0   |   0   |   0   |   0   ||   1   | DUMP  | FID comparison error - DUMP                                                  |
--------------------------------------------------------------------------------------------------------------------------------------------------
-- where '0' = OK, '1' = ERROR, x = don't care.
--       c: current.


-- TRUTH TABLE for DUMPS BASED ON PREVIOUS vs CURRENT FID COMPARISONS:
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--| FIDc /= FIDp | LK_Ac | ERRAc | LK_Bc | ERRBc | LK_Ap | ERRAp | LK_Bp | ERRBp || ERRF  | DUMP  | Comment                                                                              |
--|--------------|-------|-------|-------|-------|-------|-------|-------|-------||-------|----------------------------------------------------------------------------------------------|
--|      0       |   x   |   x   |   x   |   x   |   x   |   x   |   x   |   x   ||   0   | no    | No FID error                                                                         |
--|      1       |   1   |   x   |   1   |   x   |   x   |   x   |   x   |   x   ||   0   | no    | FID error - dump inhibited by BOTH links being down/having errors for current frame  |
--|      1       |   1   |   x   |   x   |   1   |   x   |   x   |   x   |   x   ||   0   | no    | FID error - dump inhibited by BOTH links being down/having errors for current frame  |
--|      1       |   x   |   1   |   1   |   x   |   x   |   x   |   x   |   x   ||   0   | no    | FID error - dump inhibited by BOTH links being down/having errors for current frame  |
--|      1       |   x   |   1   |   x   |   1   |   x   |   x   |   x   |   x   ||   0   | no    | FID error - dump inhibited by BOTH links being down/having errors for current frame  |
--|      1       |   x   |   x   |   x   |   x   |   1   |   x   |   1   |   x   ||   0   | no    | FID error - dump inhibited by BOTH links being down/having errors for previous frame |
--|      1       |   x   |   x   |   x   |   x   |   1   |   x   |   x   |   1   ||   0   | no    | FID error - dump inhibited by BOTH links being down/having errors for previous frame |
--|      1       |   x   |   x   |   x   |   x   |   x   |   1   |   1   |   x   ||   0   | no    | FID error - dump inhibited by BOTH links being down/having errors for previous frame |
--|      1       |   x   |   x   |   x   |   x   |   x   |   1   |   x   |   1   ||   0   | no    | FID error - dump inhibited by BOTH links being down/having errors for previous frame |
--|      1       |   0   |   0   |   0   |   0   |   0   |   0   |   0   |   0   ||   1   | DUMP  | FID error - DUMP                                                                     |
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- where '0' = OK, '1' = ERROR, x = don't care.
--       c: current, p: previous.


 
LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.ALL;

LIBRARY work;

ENTITY ERR IS 
   PORT
   (
    ERRA_in      :  IN  STD_LOGIC; -- CRC of LK_A is ok = '0'
    ERRB_in      :  IN  STD_LOGIC; -- CRC of LK_B is ok = '0'
    ERRC_in      :  IN  STD_LOGIC; -- Comparisons of CRCs is ok = '0'
    LK_A         :  IN  STD_LOGIC; -- LK_A is operational = '0'
    LK_B         :  IN  STD_LOGIC; -- LK_B is operational = '0'
    Ready_ChA    :  IN  STD_LOGIC; -- Channel A data are ready = '1'
    Ready_ChB    :  IN  STD_LOGIC; -- Channel B data are ready = '1'
    procEN_ChA   :  IN  STD_LOGIC;
    procEN_ChB   :  IN  STD_LOGIC;
    CID_ChA      :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
    CID_ChB      :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
    CRC_ChA      :  IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    CRC_ChB      :  IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    FID_ChA      :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
    FID_ChB      :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
    StatusAx     :  IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
    StatusBx     :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
    ignoreCFC    :  IN  STD_LOGIC; -- Flag dumps unecessarry when ='1'
    clk          :  IN  STD_LOGIC;
    reset        :  IN  STD_LOGIC;
    
    ValidSignal  :  OUT  STD_LOGIC;
    ReEn_Part_A  :  OUT  STD_LOGIC;
    ReEn_Part_B  :  OUT  STD_LOGIC;
    rdErrors_out :  OUT  STD_LOGIC;
    rdAddr_out   :  OUT  STD_LOGIC;
    rdpart2x_out :  OUT  STD_LOGIC;
    ERRVoltage   :  OUT  STD_LOGIC;
    ERRCFC       :  OUT  STD_LOGIC;
    ERRA         :  OUT  STD_LOGIC;
    ERRB         :  OUT  STD_LOGIC;
    ERRC         :  OUT  STD_LOGIC;
    ERRD         :  OUT  STD_LOGIC;
    ERRF         :  OUT  STD_LOGIC;
    Dump         :  OUT  STD_LOGIC
   );
END ERR;

ARCHITECTURE RTL OF ERR IS 

  SIGNAL FID_D                   :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL clr_comp                :  STD_LOGIC;
  SIGNAL ERRA_D                  :  STD_LOGIC := '0';
  SIGNAL ERRA_int                :  STD_LOGIC;
  SIGNAL ERRB_D                  :  STD_LOGIC := '0';
  SIGNAL ERRB_int                :  STD_LOGIC;
  SIGNAL ERRD_int                :  STD_LOGIC;
  SIGNAL ERRFpc                  :  STD_LOGIC;
  SIGNAL ERRFc                   :  STD_LOGIC;
  
  -- Link and Status dump signals
  SIGNAL	Dump_LK                 :  STD_LOGIC;
  SIGNAL	Dump_ST                 :  STD_LOGIC;

  -- Signals for the Read Enable chain
  SIGNAL enableRD                :  STD_LOGIC;
  SIGNAL enableRD_D              :  STD_LOGIC;
  SIGNAL enableRD_DD             :  STD_LOGIC;
  SIGNAL enRead                  :  STD_LOGIC;
  SIGNAL rdErrors                :  STD_LOGIC;
  SIGNAL rdAddr                  :  STD_LOGIC;
  SIGNAL rdpart1x                :  STD_LOGIC;
  SIGNAL rdpart2x                :  STD_LOGIC;
  SIGNAL rdEnd                   :  STD_LOGIC;
  
  SIGNAL LK_A_D                  :  STD_LOGIC := '0';
  SIGNAL LK_B_D                  :  STD_LOGIC := '0';
  
  SIGNAL ValidSignal_int         :  STD_LOGIC;
  
  SIGNAL reset_n                 :  STD_LOGIC;

  
BEGIN 


  reset_n <= NOT reset;

-- Selection of the channel to be processed based on statuses: gets the raw ERR flag coming from CRC_Check.vhd -> ValidSignal is correct all the time
PROCESS (clk)
  VARIABLE LK_status      : STD_LOGIC_VECTOR(4 DOWNTO 0);
 
  BEGIN   
    IF RISING_EDGE(clk) THEN
      LK_status      := LK_A & LK_B & ERRA_in & ERRB_in & ERRC_in;
       
      CASE LK_status IS            
        -- cases where all OK
        WHEN "00000" | "00001" =>
          ValidSignal_int <= '1'; -- choosing LK_B as shown better in PCB.
       
        -- cases where LK_B is down but LK_A is OK
        WHEN "01000" | "01001"| "01010"| "01011" =>
          ValidSignal_int <= '0';
       
        -- cases where LK_B has errors but LK_A is OK
        WHEN "00010" | "00011" =>
          ValidSignal_int <= '0';
       
        -- cases where LK_A is down but LK_B is OK
        WHEN "10000" | "10001"| "10100"| "10101" =>
          ValidSignal_int <= '1';
       
        -- cases where LK_A has errors but LK_B is OK
        WHEN "00100" | "00101" =>
          ValidSignal_int <= '1';
       
        -- cases where DUMP is requested (cases: LK_A is down and LK_B has error)
        WHEN  "10010" | "10011" | "10110" | "10111" =>
          ValidSignal_int <= '1'; -- unecessary but added for consistency.
       
        -- cases where DUMP is requested (cases: LK_B is down and LK_A has error)
        WHEN  "01100" | "01101" | "01110" | "01111" =>
          ValidSignal_int <= '0'; -- unecessary but added for consistency.
       
        -- cases where DUMP is requested (cases: LK_A and LK_B have errors)
        WHEN  "00110" | "00111" =>
          ValidSignal_int <= '1'; -- unecessary but added for consistency.
       
        -- cases where DUMP is requested (cases: LK_A and LK_B are down) 
        WHEN  "11000" | "11001" | "11010" | "11011" | "11100" | "11101" | "11110" | "11111" =>
          ValidSignal_int <= '1'; -- unecessary but added for consistency.
       
        -- cases where errors in the function - DUMP is requested 
        WHEN OTHERS =>
          ValidSignal_int <= '1'; -- unecessary but added for consistency.
       
      END CASE;
    END IF;
  END PROCESS;
  
  ValidSignal <= ValidSignal_int;

  
-- Generation of the DUMP signal based on statuses: gets a single pulse for each rising edge on ERR -> single-cycle dump requests are generated for ERR flags
PROCESS (clk)
  VARIABLE LK_status      : STD_LOGIC_VECTOR(4 DOWNTO 0);
 
 
  BEGIN   
    IF RISING_EDGE(clk) THEN
      LK_status      := LK_A & LK_B & ERRA_int & ERRB_int & ERRC_in;
         
      CASE LK_status IS            
        -- cases where all OK
        WHEN "00000" | "00001" =>
          Dump_LK     <= '0';
       
        -- cases where LK_B is down but LK_A is OK
        WHEN "01000" | "01001"| "01010"| "01011" =>
          Dump_LK     <= '0'; -- should become '1' for safety and '0' for availability
        
        -- cases where LK_B has errors but LK_A is OK
        WHEN "00010" | "00011" =>
          Dump_LK     <= '0';
        
        -- cases where LK_A is down but LK_B is OK
        WHEN "10000" | "10001"| "10100"| "10101" =>
          Dump_LK     <= '0'; -- should become '1' for safety and '0' for availability
        
        -- cases where LK_A has errors but LK_B is OK
        WHEN "00100" | "00101" =>
          Dump_LK     <= '0';
        
        -- cases where DUMP is requested (cases: LK_A is down and LK_B has error)
        WHEN  "10010" | "10011" | "10110" | "10111" =>
          Dump_LK     <= '1';
        
        -- cases where DUMP is requested (cases: LK_B is down and LK_A has error)
        WHEN  "01100" | "01101" | "01110" | "01111" =>
          Dump_LK     <= '1';
        
        -- cases where DUMP is requested (cases: LK_A and LK_B have errors)
        WHEN  "00110" | "00111" =>
          Dump_LK     <= '1';
       
        -- cases where DUMP is requested (cases: LK_A and LK_B are down) 
        WHEN  "11000" | "11001" | "11010" | "11011" | "11100" | "11101" | "11110" | "11111" =>
          Dump_LK     <= '1'; -- the empty BLECFs slots will cause a dump.
        
        -- cases where errors in the function - DUMP is requested 
        WHEN OTHERS =>
          Dump_LK     <= '1';
       
      END CASE;
    END IF;
  END PROCESS;
  
  
-- CRC, FID, CID comparison
PROCESS (clk, clr_comp)
  BEGIN
    IF (clr_comp = '1') THEN
      ERRA_int <= '0';
      ERRB_int <= '0';
      ERRC     <= '0';
      ERRD_int <= '0';
      ERRFpc   <= '0';
      ERRFc    <= '0';
    ELSIF RISING_EDGE(clk) THEN
      -- If both links are down, there will be no enableRD pulse -> ensure memorization of link status
      IF (LK_A = '1') AND (LK_B = '1') THEN
        LK_A_D <= '1';
        LK_B_D <= '1';
      END IF;
      
      IF (enableRD = '1') THEN    
        -- Generate internal ERRA and ERRB output pulses: don't repeat pulse if link has gone down in the meantime!
        IF (LK_A = '0') THEN
          IF (ERRA_in = '1') THEN
            ERRA_int <= '1';
          ELSE
            ERRA_int <= '0';
          END IF;
        END IF;
        IF (LK_B = '0') THEN
          IF (ERRB_in = '1') THEN
            ERRB_int <= '1';
          ELSE
            ERRB_int <= '0';
          END IF;
        END IF;
        
        -----------------------------------------------------------
        -- lkA vs lkB comparisons: only do if neither link is down
        -----------------------------------------------------------
        IF (LK_A = '0') AND (LK_B = '0') THEN
          -- CRC comparison: NOT DIRECTLY CONNECTED TO DUMP!
          IF CRC_ChA /= CRC_ChB THEN
            ERRC <= '1';
          ELSE
            ERRC <= '0';
          END IF;
          
          -- FID and CID comparisons: only do if neither link has errors (in addition to neither being down)
          IF (ERRA_in = '0') AND (ERRB_in = '0') THEN         
            IF FID_ChA /= FID_ChB THEN
              ERRFc <= '1';
            ELSE
              ERRFc <= '0';
            END IF;
          
            IF CID_ChA /= CID_ChB THEN
              ERRD_int <= '1';
            ELSE
              ERRD_int <= '0';
            END IF;
          END IF;
        END IF;
         
        -----------------------------------------------------------
        -- Current FID vs. previous FID comparisons: only if one of the links has no errors and is up AND one of the links had no errors in previous frame and was up for previous frame
        -----------------------------------------------------------
        IF (((ERRA_in = '0') AND (LK_A = '0')) OR ((ERRB_in = '0') AND (LK_B = '0'))) AND (((ERRA_D = '0') AND (LK_A_D = '0')) OR ((ERRB_D = '0') AND (LK_B_D = '0'))) THEN
          -- Choice: Channel A
          IF (ValidSignal_int = '0') THEN
            IF (FID_ChA /= FID_D + 1) THEN
              ERRFpc <= '1';
            ELSE
              ERRFpc <= '0';
            END IF;
           
          -- Choice: Channel B
          ELSE
            IF (FID_ChB /= FID_D + 1) THEN
              ERRFpc <= '1';
            ELSE
              ERRFpc <= '0';
            END IF;
          END IF;
          
        END IF;
        
        -- Register previous FID
        IF (ValidSignal_int = '0') THEN
          FID_D <= FID_ChA;
        ELSE
          FID_D <= FID_ChB;
        END IF;
        
        -- Register ERR and LK to have values for previous frame
        ERRA_D <= ERRA_in;
        ERRB_D <= ERRB_in;
        LK_A_D <= LK_A;
        LK_B_D <= LK_B;
      END IF;
    END IF;
  END PROCESS;

-- Generate Status Dump signal      
PROCESS (clk)
  BEGIN
    IF RISING_EDGE(clk) THEN
      Dump_ST <= ERRD_int OR ERRFpc OR ERRFc;
    END IF;
  END PROCESS;  
  
  -- NOT DIRECTLY CONNECTED TO DUMP!
  ERRVoltage <= NOT ( StatusAx(3) AND StatusAx(2) AND StatusAx(1) AND StatusAx(0) );
  -- NOT DIRECTLY CONNECTED TO DUMP!
  ERRCFC <= NOT ( StatusBx(15) AND StatusBx(14) AND StatusBx(13) AND StatusBx(12) AND StatusBx(11) AND StatusBx(10) AND StatusBx(9) AND StatusBx(8) AND
                  StatusBx(7) AND StatusBx(6) AND StatusBx(5) AND StatusBx(4) AND StatusBx(3) AND StatusBx(2) AND StatusBx(1) AND StatusBx(0) 
                  );
  
  -- Generate Status Error signals
  ERRA    <= ERRA_int;
  ERRB    <= ERRB_int;
  ERRD    <= ERRD_int;
  ERRF    <= ERRFpc OR ERRFc;

  -- Generate GLOBAL DUMP signal
  Dump <= NOT ignoreCFC AND (Dump_LK OR Dump_ST);


  
-----------------------------------
-- Read Enable chain
-----------------------------------
 
-- The initial Read Enable gets asserted if:
--  - For both links: procEN has been asserted OR link is down (LK_x = '1')
--    AND
--  - The links that are not down are Ready
PROCESS (clk)
  BEGIN 
    IF RISING_EDGE(clk) THEN        
      IF ( ((procEN_ChA = '1') OR (LK_A = '1')) AND ((procEN_ChB = '1') OR (LK_B = '1')) ) THEN
        IF    LK_A = '1' THEN
          enRead <= Ready_ChB;
        ELSIF LK_B = '1' THEN
          enRead <= Ready_ChA;
        ELSE
          enRead <= Ready_ChA AND Ready_ChB;
        END IF;
      ELSE
        enRead <= '0';
      END IF;
    END IF;    
  END PROCESS;

  
PROCESS (clk, reset_n)
  BEGIN
    IF (reset_n = '0') THEN
      rdErrors  <= '0';
      rdAddr    <= '0';
      rdpart1x  <= '0';
      rdpart2x  <= '0';
      rdEnd     <= '0';
    ELSIF (RISING_EDGE(clk)) THEN
      rdErrors  <= enRead;
      rdAddr    <= rdErrors;
      rdpart1x  <= rdAddr;
      rdpart2x  <= rdpart1x;
      rdEnd     <= rdpart2x;
    END IF;
  END PROCESS;

  ReEn_Part_A   <= rdpart1x AND NOT rdpart2x;
  ReEn_Part_B   <= rdpart2x AND NOT rdEnd;

  rdErrors_out  <= rdErrors;
  rdAddr_out    <= rdAddr;
  rdpart2x_out  <= rdpart2x;



PROCESS (clk)
  BEGIN
    IF RISING_EDGE(clk) THEN
      -- Read enable for comparisons
      IF (rdpart1x = '1') AND (rdpart2x = '0') THEN
        enableRD <= '1';
      ELSE
        enableRD <= '0';
      END IF;
    
      enableRD_D  <= enableRD;
      enableRD_DD <= enableRD_D;
    END IF;
  END PROCESS;
  
  -- Clear comparators on Reset or two cycles after the Read Enable pulse
  clr_comp <= reset OR enableRD_DD;
  
END RTL;
