------------------------------------------------
-- File         : singleDS2401_40MHz.vhd
-- Designer     : P. Vichoudis
-- Date         : 2008.08.28
------------------------------------------------
-- Description   : 
-- reads the serial number from a DS2401 which is 
-- the only slave on a 1-wire bus. The serial number
-- is valid when the associated valid signal is '1'.
--
-- Note: The clock frequency is 40Mhz.
------------------------------------------------

library ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.all;

--  Entity Declaration
ENTITY singleDS2401_40MHz IS
   PORT
   (
      clk              : in     std_logic;
      reset            : in     std_logic;   -- active high
      -----------------------
      data             : inout  std_logic;
      serialnum        : out    std_logic_vector(63 downto 0);
      serialnum_valid  : out    std_logic
   );
END singleDS2401_40MHz;

ARCHITECTURE behave OF singleDS2401_40MHz IS

--===================--
-- type declaration
--===================--
type state_type is 
(
   idle, 
   sendresetpulse,
   readpresencepulse,
   sendreadromcommand,
   readrom,
   trap
);
--===================--



--===================--
-- signal declaration
--===================--
signal datain     : std_logic;
signal forcelow   : std_logic;
--===================--

--===================--
--###################--
--###################--
--## architecture ###--
--#### begins #######--
--##### here ########--
--###################--
--###################--
--===================--

BEGIN

--===================--
process(reset,clk) 
--===================--
constant command          : std_logic_vector(7 downto 0):=x"33";
variable timer            : integer;
variable datainreg        : std_logic;
variable deviceresponded  : std_logic;
variable state            : state_type;
variable romreg           : std_logic_vector(63 downto 0);
variable i                : integer range 0 to 63;

begin

if reset='1' then

   forcelow       <='0';
   state          := idle;

   SERIALNUM_VALID   <= '0';
   SERIALNUM         <= (others => '0');
   
elsif clk'event and clk='1' then
   
   SERIALNUM_VALID   <= '0'; -- default state
   
   case state is
      
      --===================--
      when idle =>
      --===================--

         timer := 20800;
         state := sendresetpulse;
         
      --===================--
      when sendresetpulse => 
      --===================--
      
         if timer=0 then -- jump to next stage 800 clks (20us) after release 
            state := readpresencepulse; 
            timer := 20000;
            deviceresponded:='0';
         else
            case timer is
               when 20800 => forcelow <= '1'; -- force low 
               when   800 => forcelow <= '0'; -- release -- 20000 clks (500us) after
               when others => 
            end case;
            timer:=timer-1;
         end if;
      
      --===================--   
      when readpresencepulse => 
      --===================--
         
         if timer=0 then -- jump to next stage 20000 clks (500us) after resetpulse 
            
            if deviceresponded = '1' and datainreg='1' then 
               state := sendreadromcommand; 
               timer:= 3200;
               i:=0;
            else                                    
               state := sendresetpulse;
            end if;
         else
            timer := timer-1;
            if datainreg='0' then deviceresponded:='1'; end if;
         end if;
      
      --===================--
      when sendreadromcommand => --command 0x33, lsb first
      --===================--
      
         if timer=0 then 
            if i=7 then state :=readrom; i:=0; timer:=3200;
            else
            i:=i+1; timer:=3200;
            end if;
         else
            -- tSLOT=80us --
            case timer is
              ------ write bit7 ---------------------
              when  3200 => forcelow <= '1';            -- initiate  --  400 clks (10us) after  
             when  2800 => forcelow <= not command(i);  -- write bit --  400 clks (10us) after  
              when   400 => forcelow <= '0';            -- release   -- 2400 clks (60us) after  
              ---------------------------------------
             when others => 
             ---------------------------------------
         end case;   
         timer := timer-1;
         end if;
      
      --===================--
      when readrom =>
      --===================--
         
         if timer=0 then
            if i=63 then 
               state         :=trap;
               SERIALNUM_VALID   <= '1';
            else
            i:=i+1; timer:=3200;
            end if;
         else
         -- tSLOT=80us --
            case timer is 
                when 3200 => forcelow       <= '1'; -- initiate
                when 3180 => forcelow       <= '0'; -- as input  --   20 clks (500ns) after  
                when 2780 => forcelow       <= '0'; -- sample    --  400 clks ( 10us) after  
                          SERIALNUM(i)   <= datainreg;
                when others =>   
            end case;   
         timer:=timer-1;
         end if;   
      
      --===================--
      when trap =>
      --===================--
         
         SERIALNUM_VALID   <= '0';
         
   end case;
   
datainreg := datain;

end if;
end process;

DATA   <= '0' when forcelow='1' else 'Z';
datain <= 'Z' when forcelow='1' else DATA;


END behave;