---------------------------------------------------------------------------------------------------
-- FILE: DataCombining.vhd
---------------------------------------------------------------------------------------------------
-- This process will combine the acquired, by the ADC and the counter circuits, data.  
-- In addition a minimum ADC value hold circuit is used to cutoff the noise seen in the ADC value. 

-- Version 5.4
-- 10/02/2010
-- Christos Zamantzas 
---------------------------------------------------------------------------------------------------

-- v.4.8 Added rounding of the output.
-- v.4.9 'rden_out' synchronisation
-- v.5.0 Changed logic
--        - resetN was changed to reset; 
--        - beatification of the code.
-- v.5.1 Changed ieee library from ARITHM to NUMERIC 
-- v.5.2 Major change of logic
--        - Check if the new ADC value is still inside the ADC range moved to the top.
--        - Effort to remove the spikes after the capacitor charge (in CFC): 
--            - detect the reset; if the input was equal to 1 count only,
--            - and if the new output does not exceed either the previous or 128/4096 bits
--            - chose to repeat the previous output.
-- v.5.3 Major change of logic:
--            - rearrange of input pins
--            - rename of I/Os
-- v.5.4 Major change of logic
--        - changed to async reset
--        - added more signals in the reset
--        - detached the registering of the new data with their processing - gain of 40us. 
--        - rename of several internal signals

-- TODO:
--  Change logic (NOT YET!)
--        - TBD :: cut the use of the ADC result if counts are more than 16.
--        - move the limiter of values which are outside the range in the upstream block
--        - move the  the ADC noise cut-off parts in the upstream block.

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.all;
USE IEEE.STD_LOGIC_UNSIGNED.all;
USE IEEE.NUMERIC_STD.all;

ENTITY DataCombining IS

PORT(
    Din       : in  STD_LOGIC_VECTOR(19 downto 0);
    rden_in   : in  STD_LOGIC;
    ADC_Range : in  STD_LOGIC_VECTOR(11 downto 0);
    clk       : in  STD_LOGIC;
    reset     : in  STD_LOGIC;
    
    Dout      : out STD_LOGIC_VECTOR(17 downto 0);
    rden_out  : out STD_LOGIC
    );

END DataCombining;

ARCHITECTURE RTL OF DataCombining IS
    Signal Din_ADC_old   : STD_LOGIC_VECTOR(11 downto 0);
    Signal Din_ADC_new   : STD_LOGIC_VECTOR(11 downto 0);
    Signal Din_ADC_Range : STD_LOGIC_VECTOR(12 downto 0);
    Signal Din_Counts    : STD_LOGIC_VECTOR(19 downto 12);
    Signal Dout_int      : STD_LOGIC_VECTOR(19 downto 0);
    Signal Dout_int_old  : STD_LOGIC_VECTOR(19 downto 0);
    Signal rden_in_d     : STD_LOGIC;
    Signal rden_in_dd    : STD_LOGIC;
    Signal NewData       : STD_LOGIC;

begin

PROCESS (clk, reset)
    Variable Din_ADC_min : STD_LOGIC_VECTOR(11 downto 0);
    Variable Dout_ADC_a  : STD_LOGIC_VECTOR(11 downto 0);
    Variable Dout_ADC_b  : STD_LOGIC_VECTOR(12 downto 0); -- additional bit as the range can be up to 2 counts.
    
begin

  if reset = '1' then
    Dout_int      <= (OTHERS =>'0');
    Din_ADC_min   := (OTHERS =>'1');
    NewData       <= '0';
    
    Din_ADC_new   <= (OTHERS =>'0');
    Din_Counts    <= (OTHERS =>'0');
    Din_ADC_Range <= (OTHERS =>'0');
    Din_ADC_old   <= (OTHERS =>'0');
   
  elsif rising_edge(clk) then
     
     if rden_in = '1' then
       
       -- register inputs
       Din_ADC_old   <= Din_ADC_new;
       Din_ADC_new   <= Din(11 downto 0);
       Din_Counts    <= Din(19 downto 12);
       Din_ADC_Range <= '0' & ADC_Range;
       
       -- check if pre calculated ADC range is still true.
       if ADC_Range < Din(11 downto 0) then
          -- value is outside the range; choose one of the two options/strategies below:
          --Din_ADC_Range <= '0' & Din(11 downto 0); -- option 1: extend ADC range (valid for only one cycle)
          Din_ADC_new <= ADC_Range;                -- option 2: limit new ADC input value
       end if;  
       
       -- set flag that new data are available
       NewData <= '1';
     end if;
     
     -- Processing of the input data begins. 
     if NewData = '1' then
       
       -- clear flag that new data available
       NewData <= '0';
       
       -- CASE I: zero counts measured
       if Din_Counts = X"00" then
           
           -- cut-off noise by keeping the minimum value from last count.
           if Din_ADC_min > Din_ADC_old OR Din_ADC_min = X"000" then
              Din_ADC_min := Din_ADC_old;
           end if;
           
           -- calculate std output
           Dout_ADC_a := Din_ADC_min - Din_ADC_new;
           Dout_int   <= X"00" & Dout_ADC_a; 
           
           -- ERROR Correction: detect negative results
           if Din_ADC_min < Din_ADC_new then 
              Dout_int <= (OTHERS =>'0'); 
           end if;
           
       -- CASE II/III: more than zero counts measured
       else
           -- need to set the ADC_min after the capacitor recharge (count)
           Din_ADC_min := Din_ADC_new;  
           
           -- calculate std output 
           Dout_ADC_b := Din_ADC_old + Din_ADC_Range - Din_ADC_new; 
           Dout_int <= ((Din_Counts - '1') & X"000") + Dout_ADC_b; 
           
           -- ERROR Correction: the result is compensating for missing bits in the full range
           if Din_Counts = X"01" and Dout_ADC_b < X"80" and Dout_int_old < X"80" then
             Dout_int <= Dout_int_old; -- repeat previous value
           end if;
           
       end if;
       
     end if;
    
  end if;
  
end process;

-- Delay the rden_in signal (to match the data output delay)
PROCESS (clk)
begin
    if rising_edge(clk) then
        rden_in_d  <= rden_in;
        rden_in_dd <= rden_in_d;
        rden_out   <= rden_in_dd;
    end if;
end process;

-- Rounding of the output by two LSB. 
--   Result is 18 bits (i.e. 8 bits representing the counts and 10 bits for its fraction).
PROCESS (clk)
begin
   if rising_edge(clk) then
     Dout_int_old <= Dout_int;
     Dout         <= Dout_int(19 downto 2);
     if Dout_int(1 downto 0) = "11" or Dout_int(1 downto 0) = "10" then
       Dout <= Dout_int(19 downto 2) + '1';
     end if;
   end if; 
end process;

end rtl;
