-- ===================================================================================
-- File Name: output_alignment.vhd
-- Author: Christos Zamantzas
-- Version: 15
-- ===================================================================================
-- Description: Data arrive always in pairs.
--              each word has two RSs of the same four channels.
--   assume: B,A the two nibbles of the first word.  Ch:4-1 RS(odd), Ch:4-1 RS(even)
--           D,C    -//-      -//-     second word.  Ch:8-5 RS(odd), Ch:8-5 RS(even)
--   the output should become: 
--           C,A :: Ch:8-5 RS(even) , Ch:4-1 RS(even)
--           D,B :: Ch:8-5 RS(odd)  , Ch:4-1 RS(odd)
-- ===================================================================================
-- changes in ver.06: working version for the new structure.
-- changes in ver.07: cleanup and reorganisation of processes. 
--                   modification of forwarding of pending data
-- changes in ver.08: given priority to pending data.
--                   added signal to hold the address for the Sel_out_int of pending data.
-- changes in ver.09: changed the address for the Sel_out_int for the pending data.
-- changes in ver.10: corrected semantic error when specifying odd and even for Ch:8-5.
-- changes in ver.11: added one cycle delay to Sel_out_int and rden_out_int
-- changes in ver.12: removed unused Sel_pend signal.
-- changes in ver.13: correction of semantic error on the output sequence for rden_out_int.
-- changes in ver.14: regression to version 10 + added version 12 changes.
-- changes in ver.15: added one cycle delay to Sel_out_int and rden_out_int


-- ===================================================================================
-- TODO:
-- ===================================================================================

LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY output_alignment IS
   PORT
   (
      Data     : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
    
      Sel      : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
      rden     : IN STD_LOGIC;
      clk      : IN STD_LOGIC;
    
      Data_out : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
      Sel_out  : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
      rden_out : OUT STD_LOGIC
   );

END output_alignment;

ARCHITECTURE RTL OF output_alignment IS

   SIGNAL Data_int   : STD_LOGIC_VECTOR (7 DOWNTO 0);
   SIGNAL Sel_int    : STD_LOGIC_VECTOR (3 DOWNTO 0);
   SIGNAL Sel_intp   : STD_LOGIC_VECTOR (3 DOWNTO 0);
   SIGNAL Data_intA  : STD_LOGIC_VECTOR (3 DOWNTO 0);
   SIGNAL Data_intB  : STD_LOGIC_VECTOR (3 DOWNTO 0);
   SIGNAL Data_intD  : STD_LOGIC_VECTOR (3 DOWNTO 0);
   SIGNAL Data_intBp : STD_LOGIC_VECTOR (3 DOWNTO 0);
   SIGNAL Data_read  : STD_LOGIC;
   SIGNAL Data_pend  : STD_LOGIC;
   SIGNAL Sel_out_int : STD_LOGIC_VECTOR (3 DOWNTO 0);
   SIGNAL rden_out_int  : STD_LOGIC;


BEGIN

PROCESS (clk)

BEGIN
   IF rising_edge(clk) THEN
    
      rden_out_int  <= '0';                                      -- flag: no new output to read/clear from previous read enable - Default value 
      Sel_intp      <= Sel_int;                                  -- delay 1 cycle 
    
      -- Process pending data if available.
      if Data_pend = '1' then
         Data_pend     <= '0';                                   -- clear flag; the data are being processed.
         Data_out      <= Data_intD & Data_intBp;                -- output data; D,B::odd RSs of all channels.
         rden_out_int  <= '1';                                   -- flag: new output to read
         Sel_out_int   <= Sel_intp;                              -- addr: new output to read
      end if;
    
      -- Process new data if available.
      if Data_read = '1' then
         Data_read     <= '0';                                   -- clear flag; the data are being processed.
         if Sel_int(0) = '0' then
         -- 1st part: data coming for channels 1-4.
            Data_intA  <= Data_int (3 downto 0);                 -- A: even RSs (0, 2, 4, 6, 8, 10)
            Data_intB  <= Data_int (7 downto 4);                 -- B: odd  RSs (1, 3, 5, 7, 9, 11)
         else
         -- 2nd part: data coming for channels 5-8.
           -- Register Data for later use
            Data_intD  <= Data_int (7 downto 4);                 -- D: odd  RSs (1, 3, 5, 7, 9, 11)
            Data_intBp <= Data_intB;                             -- delay B nibble one cycle more 
            Data_pend  <= '1';                                   -- flag: Data are pending for the output
            -- output even RSs for all channels.
            Data_out     <= Data_int (3 downto 0) & Data_intA;   -- output data; C,A::even RSs of all channels.
            rden_out_int <= '1';                                 -- flag: new output to read
            Sel_out_int  <= Sel_intp;                            -- addr: new output to read
         end if;
       
      end if;
    
      -- Register new input data if available.
      if rden = '1' then
         Data_int  <= Data;                                  -- register input data
         Data_read <= '1';                                   -- flag: new Data have been read.        
         Sel_int   <= Sel;                                   -- register inputs
      end if;
      
      -- Register outputs (except Data_out to give one cycle diff)
      Sel_out  <= Sel_out_int;
      rden_out <= rden_out_int;
    
   END IF;
END PROCESS;

END RTL;